# Test: minors of banded symmetric Toeplitz matrices
# as skew Schur polynomials expressed via Dickson--Zhukovsky variables.

def PRing(letter, n):
   names = [letter + str(k) for k in range(n)]
   return PolynomialRing(QQ, names)

def vars(letter, n):
   return PRing(letter, n).gens()

def prod_dif(x, j):
   return prod(x[j] - x[k] for k in range(j) + range(j + 1, len(x)))

def homz(z, m):
   n = len(z)
   return sum(chebyshev_U(n + m - 1, z[j] / 2) / prod_dif(z, j) for j in range(n))

def skewschurz(z, la, mu):
   F = z[0].parent()
   m = len(la)
   muext = mu + [0] * (m - len(mu))
   JT = matrix(F, m, m)
   for j in range(m):
      for k in range(m):
         p = la[j] - muext[k] - j + k
         JT[j, k] = homz(z, p) if p >= 0 else 0
   return JT.det()

def mul_polynomial_by_binomial(acoefs, b):
   alen = len(acoefs)
   midcoefs = [acoefs[j] * b + acoefs[j - 1] for j in range(1, alen)]
   return [b * acoefs[0]] + midcoefs + [acoefs[alen - 1]]

def elem_pols(t):
   m = len(t)
   result = [1]
   for j in range(m):
      result = mul_polynomial_by_binomial(result, t[j])
   return list(reversed(result))

def elemz_via_lincomb_elem(z):
   n = len(z)
   elem_z = elem_pols(z)
   result = [0] * (2 * n + 1)
   for m in range(2 * n + 1):
      m1 = max(0, m - n)
      m2 = floor(m / 2)
      result[m] = sum(binomial(n - m + 2 * k, k) * elem_z[m - 2 * k] for k in range(m1, m2 + 1))
   return result

def banded_symmetric_toeplitz_matrix(z, m):
   n = len(z)
   F = z[0].parent()
   minmn = min(m, n + 1)
   difmn = max(m - n - 1, 0)
   elems0 = elemz_via_lincomb_elem(z)
   elems1 = [elems0[k] * ((-1) ** k) for k in range(2 * n + 1)]
   elems2 = elems1[n : ]
   elems3 = elems2[ : minmn] + [F.zero()] * difmn
   return matrix.toeplitz(elems3, elems3[1 : ], F)

def banded_symmetric_toeplitz_submatrix(z, m, rows, cols):
   r = len(rows)
   bigmatrix = banded_symmetric_toeplitz_matrix(z, m)
   elems = [[bigmatrix[r, s] for s in cols] for r in rows]
   F = z[0].parent()
   submat = matrix(F, elems)
   return submat

def banded_symmetric_toeplitz_minor(z, m, rows, cols):
   T = banded_symmetric_toeplitz_submatrix(z, m, rows, cols)
   F = z[0].parent()
   return F(det(T))

def complement(m, mylist):
   setmylist = set(mylist)
   return [j for j in range(m) if j not in setmylist]

def banded_symmetric_toeplitz_minor_striked(z, m, xi, eta): 
   xi1 = complement(m, xi)
   eta1 = complement(m, eta)
   return banded_symmetric_toeplitz_minor(z, m, xi1, eta1)

def oursign(m, n, xi, eta):
   k = n * (m - len(xi)) + sum(xi) + sum(eta)
   return (-1) ** k

def difrev(xi):
   return list(reversed([xi[j] - j for j in range(len(xi))]))

def ourlambda(m, n, xi, eta):
   return [m - len(xi)] * n + difrev(xi)

def ourmu(m, n, xi, eta):
   return difrev(eta)

def banded_symmetric_toeplitz_minor_formula(z, m, xi, eta):
   n = len(z)
   la = ourlambda(m, n, xi, eta)
   mu = ourmu(m, n, xi, eta)
   F = z[0].parent()
   return F(oursign(m, n, xi, eta) * skewschurz(z, la, mu))

def test_banded_symmetric_toeplitz_minor_pol(n, m, xi, eta, verbos):
   if verbos >= 2:
      print('test_banded_symmetric_toeplitz_minor_pol,')
      print('n = %d, m = %d, xi = %s, eta = %s' % (n, m, str(xi), str(eta)))
   z = vars('z', n)
   lhs = banded_symmetric_toeplitz_minor_striked(z, m, xi, eta)
   rhs = banded_symmetric_toeplitz_minor_formula(z, m, xi, eta)
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_banded_symmetric_toeplitz_minor_pol(nmax, mmax, verbos):
   if verbos >= 1:
      print('big_test_banded_symmetric_toeplitz_minor_pol, nmax = %d, mmax = %d' % (nmax, mmax))
   counttests = 0
   countnonzero = 0
   tasks = []
   for n in range(1, nmax + 1):
      for m in range(mmax + 1):
         for d in range(m + 1):
            subs = list(Subsets(range(m), m - d))
            for indxi in range(len(subs)):
               for indeta in range(len(subs)):
                  xi = list(subs[indxi])
                  eta = list(subs[indeta])
                  tasks += [(n, m, xi, eta)]
   if verbos >= 1:
      print('number of tasks = %d' % len(tasks))
   result = all(test_banded_symmetric_toeplitz_minor_pol(n, m, xi, eta, verbos) for (n, m, xi, eta) in tasks)
   print('total result = ' + str(result))

#print(test_banded_symmetric_toeplitz_minor_pol(2, 3, [0], [1], 3))
#print(test_banded_symmetric_toeplitz_minor_pol(4, 6, [1, 3], [0, 2], 3))

big_test_banded_symmetric_toeplitz_minor_pol(3, 7, 1)

